package pl.netanel.swt.matrix.snippets;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import pl.netanel.swt.matrix.*;

/**
 * Sorting by columns.
 */
/*
 * Make sure ascending.png and descending.png is on your class path.
 */
public class S0055_SortingByColumns {

  public static void main(String[] args) throws Exception {
    Shell shell = new Shell();
    shell.setText(title);
    shell.setLayout(new GridLayout(2, false));
    Display display = shell.getDisplay();

    // Model
    final ArrayList<String[]> list = new ArrayList<String[]>();
    list.add(new String[]{"Task 1", "high"});
    list.add(new String[]{"Task 2", "medium"});
    list.add(new String[]{"Task 3", "low"});
    list.add(new String[]{"Task 4", "high"});
    list.add(new String[]{"Task 5", "medium"});

    // Sorted list holder
    final ArrayList<String[]> sorted = new ArrayList<String[]>();
    sorted.addAll(list);

    // Sorting images and state
    final Image sortAsc = new Image(display, 9, 5);
    GC gc = new GC(sortAsc);
    gc.setAntialias(SWT.ON);
    gc.setBackground(display.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
    gc.fillRectangle(0, 0, 9, 5);
    gc.setBackground(display.getSystemColor(SWT.COLOR_DARK_GRAY));
    gc.setAntialias(SWT.ON);
    gc.fillPolygon(new int[]{5, 0, 9, 5, 0, 5});
    gc.dispose();

    final Image sortDesc = new Image(display, 9, 5);
    gc = new GC(sortDesc);
    gc.setAntialias(SWT.ON);
    gc.setBackground(display.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
    gc.fillRectangle(0, 0, 9, 5);
    gc.setBackground(display.getSystemColor(SWT.COLOR_DARK_GRAY));
    gc.setAntialias(SWT.ON);
    gc.fillPolygon(new int[]{5, 5, 9, 0, 0, 0});
    gc.dispose();

    final int[] direction = new int[]{0, 0};

    // Matrix
    final Matrix<Integer, Integer> matrix = new Matrix<Integer, Integer>(shell,
        SWT.V_SCROLL);
    matrix.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));

    Axis<Integer> axisY = matrix.getAxisY();
    final Section<Integer> bodyY = axisY.getBody();
    bodyY.setCount(list.size());
    bodyY.setDefaultResizable(true);
    bodyY.setDefaultMoveable(true);
    axisY.getHeader().setVisible(true);

    Axis<Integer> axisX = matrix.getAxisX();
    axisX.getBody().setCount(2);
    axisX.getHeader().setDefaultCellWidth(16);
    axisX.getHeader().setVisible(true);

    // Paint data text in the body zone
    matrix.getBody().replacePainterPreserveStyle(
        new Painter<Integer, Integer>(Painter.NAME_CELLS) {
          @Override
          public void setupSpatial(Integer indexX, Integer indexY) {
            text = sorted.get(indexY)[indexX].toString();
          }
        });

    // Paint text and sorting image in the column headers zone
    Painter<Integer, Integer> columnHeaderPainter = new Painter<Integer, Integer>(
        Painter.NAME_CELLS) {
      @Override
      public void setupSpatial(Integer indexX, Integer indexY) {
        text = indexX.toString();

        int column = indexX;
        image = direction[column] == 0 ? null
                                       : direction[column] == 1 ? sortAsc : sortDesc;
      }
    };

    matrix.getHeaderX().replacePainterPreserveStyle(columnHeaderPainter);
    Style style = columnHeaderPainter.style;
    style.imageAlignX = SWT.RIGHT;
    style.imageAlignY = SWT.CENTER;
    style.imageMarginX = 5;

    // Change sorting on mouse down in a column header
    matrix.getHeaderX().addListener(SWT.MouseDown, new Listener() {
      @Override
      public void handleEvent(Event e) {
        // AxisItem<Y> item0 = matrix.getAxisY().getItemByDistance(e.y);
        AxisItem<Integer> itemX = matrix.getAxisX().getMouseItem();
        if (itemX == null) return;
        final int column = itemX.getIndex();

        int previousDirection = direction[column];
        for (int i = 0; i < direction.length; i++) {
          direction[i] = 0;
        }
        direction[column] = previousDirection <= 0 ? 1 : -1;

        Collections.sort(sorted, new Comparator<String[]>() {
          @Override
          public int compare(String[] o1, String[] o2) {
            return o1[column].compareTo(o2[column]) * direction[column];
          }
        });
        matrix.redraw();
      }
    });

    // Unbind column selection on click
    matrix.getHeaderX().unbind(Matrix.CMD_SELECT_COLUMN, SWT.MouseDown, 1);
    matrix.getHeaderX().unbind(Matrix.CMD_FOCUS_LOCATION, SWT.MouseDown, 1);

    shell.setBounds(400, 200, 600, 400);
    shell.open();
    while (!shell.isDisposed()) {
      if (!display.readAndDispatch()) {
        display.sleep();
      }
    }
    sortAsc.dispose();
    sortDesc.dispose();
  }

  static Image readImage(String path) throws IOException {
    FileInputStream stream = new FileInputStream(path);
    Image image = new Image(Display.getCurrent(), new ImageData(stream));
    stream.close();
    return image;
  }

  // Meta data
  static final String title = "Sorting by columns";
  static final String instructions = "";
}