/*
 * Copyright (c) 2011-2014. netanel.pl.
 * All rights reserved. This source code and the accompanying materials
 * are made available under the terms of the EULA v1.2
 * which accompanies this distribution, and is available at
 * http://www.netanel.pl/swt-matrix/EULA.html
 **************************************************************************************************/

package pl.netanel.swt.matrix.snippets;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.*;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.*;
import pl.netanel.swt.matrix.*;
import pl.netanel.swt.matrix.Matrix.DropPosition;

/**
 * Custom drag and drop.
 */
@SuppressWarnings("ALL")
public class S0911_CustomDragAndDrop {
  Matrix<Integer, Integer> matrix;
  Painter<Integer, Integer> dragPainterX;
  Cursor cursorWE = Display.getDefault().getSystemCursor(SWT.CURSOR_SIZEWE);
  Cursor cursorNS = Display.getDefault().getSystemCursor(SWT.CURSOR_SIZENS);
  NumberCollection<Integer> dragged;

  public S0911_CustomDragAndDrop() {
    Shell shell = new Shell();
    shell.setText(title);
    shell.setLayout(new FillLayout());
    shell.setBounds(400, 200, 600, 400);

    matrix = new Matrix<Integer, Integer>(shell, SWT.H_SCROLL);
    final Axis<Integer> axisX = matrix.getAxisX();
    final Axis<Integer> axisY = matrix.getAxisY();

    final Section<Integer> bodyX = axisX.getBody();
    final Section<Integer> bodyY = axisY.getBody();
    bodyX.setCount(10);
    bodyY.setCount(10);
    bodyX.setDefaultResizable(true);

    axisX.getHeader().setVisible(true);
    axisY.getHeader().setVisible(true);

    axisX.setFrozenHead(6);
    axisY.getHeader().setCount(2);

    matrix.getHeaderX().setMerged(0, 3, 0, 1, true);

    dragPainterX = matrix.getPainter(Painter.NAME_DRAG_ITEM_X);

    // Modify the column selection logic to prevent selection when start moving
    // so that multiple column can be selected and dragged
    matrix.unbind(Matrix.CMD_SELECT_COLUMN, SWT.MouseDown, 1);
    matrix.addListener(SWT.MouseDown, new Listener() {
      @Override
      public void handleEvent(Event e) {
        if (e.stateMask != 0) return;
        AxisItem<Integer> itemY = axisY.getItemByViewportDistance(e.y);
        if (itemY == null || !itemY.getSection().equals(axisY.getHeader())) return;
        AxisItem<Integer> itemX = axisX.getItemByViewportDistance(e.x);
        if (itemX == null || !itemX.getSection().equals(bodyX)) return;

        // Check if not over resize area
        int[] b = axisX.getCellBound(itemX);
        int w = axisX.getResizeOffset();
        boolean inResizeArea = e.x < b[0] + w || b[0] + b[1] - w < e.x;
        if (!inResizeArea && !bodyX.isSelected(itemX.getIndex())) {
          bodyX.setSelected(0, bodyX.getCount() - 1, false);
          bodyX.setSelected(itemX.getIndex(), true);
          axisX.setFocusItem(itemX);
        }
      }
    });

    int operations = DND.DROP_MOVE;
    Transfer[] types = new Transfer[] {TextTransfer.getInstance()};
    final DragSource ds = new DragSource(matrix, operations);
    ds.setTransfer(types);
    ds.addDragListener(new DragSourceAdapter() {
      public void dragStart(DragSourceEvent event) {
        boolean enabled = isDragPainterEnabled(event.x, event.y, false);
        if (enabled) {
          dragged = axisX.getBody().getSelected().copyNumberCollection();
          dragPainterX.setEnabled(true);
          matrix.redraw();
        } else {
          event.doit = false;
        }
      }

      @Override
      public void dragSetData(DragSourceEvent event) {
        event.data = "a";
      }
    });

    final DropTarget dt = new DropTarget(matrix, operations);
    dt.setTransfer(types);
    dt.addDropListener(new DropTargetAdapter() {
      @Override
      public void drop(DropTargetEvent event) {
        TextTransfer transfer = TextTransfer.getInstance();
        if (transfer.isSupportedType(event.currentDataType)) {
          Point p = matrix.toControl(event.x, event.y);
          AxisItem<Integer> item = axisX.getItemByViewportDistance(p.x);
          if (item != null) {
            int[] bound = axisX.getCellBound(item);
            if (bound != null) {
              DropPosition dropPosition = p.x > bound[0] + bound[1] / 2 ? DropPosition.AFTER : DropPosition.BEFORE;
              bodyX.getOrder().move(dragged, dropPosition, item.getIndex());
            }
          }
          dragPainterX.setEnabled(false);
          matrix.refresh();
          finishDrag(event.x, event.y);
        }
      }

      @Override
      public void dropAccept(DropTargetEvent event) {
        boolean enabled = isDragPainterEnabled(event.x, event.y, true);
        if (enabled) {
          dragPainterX.setData(event);
        } else {
          event.detail = DND.DROP_NONE;
        }
        dragPainterX.setEnabled(enabled);
        matrix.redraw();
      }

      @Override
      public void dragOver(DropTargetEvent event) {
        dragPainterX.setData(event);
        dragPainterX.setEnabled(isDragPainterEnabled(event.x, event.y, true));
        matrix.redraw();
      }
    });

    shell.addListener(SWT.Dispose, new Listener() {
      @Override
      public void handleEvent(Event event) {
        dt.dispose();
        ds.dispose();
      }
    });

    shell.open();
    Display display = shell.getDisplay();
    while (!shell.isDisposed()) {
      if (!display.readAndDispatch()) {
        display.sleep();
      }
    }
  }


  private boolean isDragPainterEnabled(int x, int y, boolean map) {
    Point p = map ? matrix.toControl(x, y) : new Point(x, y);
    if (!matrix.getHeaderX().containsLocation(p.x, p.y)) return false;
    AxisItem<Integer> item = matrix.getAxisX().getItemByViewportDistance(p.x);
    Cursor cursor = matrix.getCursor();
    boolean isResizing = cursor != null && (cursor.equals(cursorNS) || cursor.equals(cursorWE));
    return item != null && item.getIndex() < 5 && !isResizing;
  }

  private void finishDrag(int x, int y) {
    /**
     * Drag detect prevents the mouse up event to occur.
     * With the current implementaiton of martix we have to enforce it to make it work correctly.
     * It is going to be obsolte in the future.
     */
    Event e = new Event();
    e.type = SWT.MouseUp;
    e.button = 1;
    e.x = x;
    e.y = y;
    e.widget = matrix;
    matrix.getDisplay().post(e);
  }

  public static void main(String[] args) {
    new S0911_CustomDragAndDrop();
  }

  // Meta data
  static final String title = "Custom drag and drop";
  static final String instructions = "";
}